#! /usr/bin/perl -wT

use strict ;

do ("./lib.pl") || die( "could not include subroutine library ($!)" ) ;
our $DOCTYPE ;
our $WIKI_ROOT ;
our $WIKI_PAGE_FILENAME_RE ;
# our $WIKI_NAME_RE ;
# our $WIKI_EXT ;
our $SECS_PER_DAY ;
our $WIKI_CHANGES_ACTION ;
our $FRONT_PAGE ;

($ENV{"REQUEST_METHOD"} eq "GET") || die( "script must be accessed by GET" ) ;
my $queryString = $ENV{"QUERY_STRING"} ;
my %query = &parse( $queryString ) ;
my $days = $query{"days"} ;

if (defined( $days ))
{
	&query( $days ) ;
}
else
{
	&form() ;
}

sub form
{
	my $frontUrl = &viewUrl( $FRONT_PAGE ) ;
	print "Content-type: text/html\n" ;
	print "\n" ;
	print <<EOF ;
$DOCTYPE
<html>
<head>
<title>Changes in the Last N Days</title>
</head>
<body>
<h1>Changes in the Last N Days</h1>
<hr>
<p>This form can be used to query the wiki for a list of pages which have changed recently;  
specify the age in days up to which a change should be considered 'recent'.</p>
<form method="GET" action="$WIKI_CHANGES_ACTION">
<input type="text" name="days" value="1">
<input type="submit">
<input type="reset">
</form>
<hr>
<p><a href="$frontUrl">$FRONT_PAGE</a></p>
</body>
</html>
EOF
}

sub query
{
	my $days = pop @_ ; # legal? necessary? desirable? probably none of the above.
	print "Content-type: text/html\n" ;
	print "\n" ;
	print <<EOF ;
$DOCTYPE
<html>
<head>
<title>Changes in the Last $days Days</title>
</head>
<body>
<h1>Changes in the Last $days Days</h1>
<hr>
EOF
	my @changes = &changes( $days ) ;
	print "<ul>\n" ;
	foreach my $page (@changes)
	{
		my $url = viewUrl( $page ) ;
		my $mtime = pageMtime( $page ) ;
		my $mtimestr = gmtime( $mtime ) ;
		print "<li><a href=\"$url\">$page</a> ($mtimestr GMT)</li>\n" ;
	}
	print "</ul>\n" ;
	my $frontUrl = &viewUrl( $FRONT_PAGE ) ;
	print <<EOF ;
<hr>
<p><a href="$frontUrl">$FRONT_PAGE</a></p>
</body>
</html>
EOF
}

sub changes # returns a list of page names, sorted by date
{
	my $days = shift @_ ;
	# could use opendir, could use glob, could use backticks
	opendir( DIR, $WIKI_ROOT ) ;
	my @files = readdir( DIR ) ;
	my @changes ;
	my $deadline = time() - ($days * $SECS_PER_DAY) ;
	foreach my $file (@files)
	{
		if (($file =~ /$WIKI_PAGE_FILENAME_RE/) && (mtime( ($WIKI_ROOT . $file) ) ge $deadline))
		{
			my $page = $1 ;
			push( @changes, $page ) ;
		}
	}
	@changes = sort { pageMtime( $b ) <=> pageMtime( $a ) } @changes ;
	return @changes ;
}
