#! /usr/bin/perl -wT

use strict ;

do ("./lib.pl") || die( "could not include subroutine library ($!)" ) ;
our $DOCTYPE ;
our $WIKI_VIEW_BASE ;
our $WIKI_EDIT_BASE ;
our $EMBEDDED_WIKI_NAME_RE ;
our $EMBEDDED_URL_RE ;
our @KEY_PAGES ;

# for running from command line
if (!defined( $ENV{"REQUEST_METHOD"} ))
{
	$ENV{"REQUEST_METHOD"} = "GET" ;
	$ENV{"QUERY_STRING"} = "page=FrontPage" ;
}

($ENV{"REQUEST_METHOD"} eq "GET") || die( "script must be accessed by GET" ) ;
my $queryString = $ENV{"QUERY_STRING"} ;
my %query = &parse( $queryString ) ;
my $page = &cleanPage( $query{"page"} ) ;
my $pageFile = &pageFile( $page ) ;
open( PAGE, "< " . $pageFile ) || die( "could not open page file" ) ;
print "Content-type: text/html\n" ;
print "\n" ;
print <<EOF ;
$DOCTYPE
<html>
<head>
<title>$page</title>
</head>
<body>
<h1>$page</h1>
<hr>
EOF
$/ = "" ; # paragraph mode; note that we don't ever leave it again
while (my $block = <PAGE>)
{
	&format( $block ) ;
}
close( PAGE ) ;
my $editUrl = &editUrl( $page ) ;
my $backlinksUrl = &backlinksUrl( $page ) ;
my $sourceUrl = &sourceUrl( $page ) ;
my $validateUrl = &validateUrl( $page ) ;
print <<EOF ;
<hr>
<p>
| <a href="$editUrl">Edit</a>
| <a href="$backlinksUrl">Backlinks</a>
| <a href="$sourceUrl">Source</a>
| <a href="$validateUrl">Validate</a> 
|
EOF
foreach my $keyPage (@KEY_PAGES)
{
	my $keyUrl = &viewUrl( $keyPage ) ;
	print "| <a href=\"$keyUrl\">$keyPage</a>\n" ;
}
my $mtimestr = gmtime( pageMtime( $page ) ) ;
print <<EOF ;
|
| Last Modified $mtimestr GMT
|
</p>
</body>
</html>
EOF

# below be scary formatting related things
# step one in refactoring it: extract to method
# step two will be 'hit with cattle-prods until docile'

sub format
{
	my $block = shift @_ ;
	if (substr( $block, 0, 1 ) eq "-")
	{
		list( $block ) ;
	}
	else
	{
		paragraph( $block ) ;
	}
}

sub list
{
	my $list = shift @_ ;
	my @lines = split( "\n", $list ) ;
	print "<ul>\n" ;
	foreach my $line (@lines)
	{
		if (substr( $line, 0, 1 ) eq "-")
		{
			print "<li>" ;
			$line =~ /-(?:\s*)?(.*)/ ; # UUUHHHH!
			$line = $1 ;
		}
		text( $line ) ;
		print "\n" ;
	}
	print "</ul>\n" ;
}

sub paragraph
{
	my $para = shift @_ ;
	print "<p>" ;
	text( $para ) ;
	print "</p>\n" ;
}

sub text
{
	my $text = shift @_ ;
	$text =~ s/</&lt;/g ;
	my @words = split( " ", $text ) ; # actually splits on whitespace
	foreach my $word (@words)
	{
		if (substr( $word, 0, 1 ) eq "_")
		{
			print "<em>" ;
			$word = substr( $word, 1 ) ;
		}
		elsif (substr( $word, 0, 1 ) eq "*")
		{
			print "<strong>" ;
			$word = substr( $word, 1 ) ;
		}
		my $trailingTag ;
                if (substr( $word, -1, 1 ) eq "_")
                {
                        $trailingTag = "</em>" ;
                        $word = substr( $word, 0, -1 ) ;
                }
                elsif (substr( $word, -1, 1 ) eq "*")
                {
                        $trailingTag = "</strong>" ;
                        $word = substr( $word, 0, -1 ) ;
                }
		if ($word =~ /$EMBEDDED_WIKI_NAME_RE/)
		{
			my $pre = $1 ;
			my $name = $2 ;
			my $post = $3 ;
			if (&exists( $name ))
			{
				my $href = $WIKI_VIEW_BASE . $name ;
				print "$pre<a href=\"$href\">$name</a>$post " ;
			}
			else
			{
				my $href = $WIKI_EDIT_BASE . $name ;
				print "$pre$name<a href=\"$href\">?</a>$post " ;
			}
		}
		elsif ($word =~ /$EMBEDDED_URL_RE/)
		{
                        my $pre = $1 ;
                        my $url = $2 ;
                        my $post = $3 ;
			print "$pre<a href=\"$url\">$url</a>$post " ;
		}
		else
		{
			print "$word " ;
		}
		if (defined( $trailingTag ))
		{
			print $trailingTag ;
		}
	}
}
